//*****************************************************************************
//* lcdif.h | lcdif.c
//*
//* Info: 			Treiberschicht zur Ansteuerung des LH155-Displaycontrollers
//*
//* Author(s): 		Fabian Cordes
//* Copyright: 		(c) 2009 Projekt ELE21 
//*
//*****************************************************************************

// include guards
#ifndef __LCDIF_H__
#define __LCDIF_H__

#include <avr/io.h>
#include <stdint.h>

#include "lcdsettings.h"

// Global vars
// drawing buffer
static uint8_t lineBuffer[LCD_LINE_LENGTH];


// Prototypes for driver level functions
void initLCD (void);
void writeLCDData (uint8_t dataByte);
void writeLCDCtrl (uint8_t dataByte, uint8_t ctrlByte);

// Control registers as specified by datasheet //
#define LCD_SET_X_ADDRESS 		0x0
// 0x1h reserved - setting higher 4 bit
#define LCD_SET_Y_ADDRESS 		0x2
// 0x3h reserved - setting higher 4 bit
#define LCD_SET_START_ADDRESS 	0x4
// 0x5h reserved - setting higher 4 bit
#define LCD_SET_NLINEALT 		0x6
// 0x7h reserved - setting higher 4 bit
#define LCD_DISPCTRL1 			0x8
#define LCD_DISPCTRL2 			0x9
#define LCD_INCCTRL1 			0xA
#define LCD_PWRCTRL1 			0xB
#define LCD_PWRCTRL2 			0xC
#define LCD_PWRCTRL3 			0xD
#define LCD_REFLAG1 			0xE
#define LCD_INTREADADDR1 		0xF

// Flags as specified by datasheet
// Display control register (1)
#define LCD_DCTRL1_TOGGLE 		0
#define LCD_DCTRL1_ALLOFF		1
#define LCD_DCTRL1_ALLON		2
#define LCD_DCTRL1_SEGON		3
// Display control register (2)
#define LCD_DCTRL2_REF			0
#define LCD_DCTRL2_SWAP			1
#define LCD_DCTRL2_NLIN			2
#define LCD_DCTRL2_REV			3
// Increment control register (1)
#define LCD_ICTRL1_INCX			0
#define LCD_ICTRL1_INCY			1
#define LCD_ICTRL1_INCMODE		2
// Power control register (1)
#define LCD_PCTRL1_ACL			0
#define LCD_PCTRL1_PWRON		1
#define LCD_PCTRL1_HALT			2
#define LCD_PCTRL1_BIAS			3
// Power control register (2)
#define LCD_PCTRL2_DVOL0		0
#define LCD_PCTRL2_DVOL1		1
#define LCD_PCTRL2_DVOL2		2
#define LCD_PCTRL2_DVOL3		3
// Power control register (3)
#define LCD_PCTRL3_BS0			0
#define LCD_PCTRL3_BS1			1
#define LCD_PCTRL3_DUTY0		2
#define LCD_PCTRL3_DUTY1		3
// RE register
#define LCD_RECTRL_RE			0


#endif // close include guards
