//****************************************************************************
//* gfx.c
//*
//* Info: 			Grafikausgabe-Subsystem
//*
//* Author(s): 		Fabian Cordes
//* Copyright: 		(c) 2009 Projekt ELE21 
//*
//*****************************************************************************


#include "avr/pgmspace.h"
#include "gfx.h"
#include "gfxmsg.h"
#include "panels.h"


// Typedef for function pointers
typedef void (*panelCallbackPtr) (uint8_t msgByte, uint8_t dataByte);


// Comprehensive list of available callbacks for each panel
const panelCallbackPtr panelCallbacks[GFX_MAX_PANEL_COUNT] PROGMEM =
{
	&panelCallback0,	// Boot
	&panelCallback1		// Main
	//..
	//TODO: Insert future panel handlers here
};


// Pointer to callback of active panel
static panelCallbackPtr activePanels [GFX_MAX_ACTIVE_PANEL_COUNT];
// Count of active panels
static uint8_t panelCount;

// current line
static uint8_t currentLine;


// function prototypes
void dispatchBuffer ( void );
static panelCallbackPtr getPanelCallback (uint8_t panelId);

void initGFXSubsys ()
{
	// Fill array of callback functions
//	panelCallbacks[0] = (void*) &panelCallback0;
//	panelCallbacks[1] = (void*) &panelCallback1;
	//...
	// TODO: Insert future panel handlers here

	// Start drawing at first line
	currentLine = 0;

	// Full sails ahead
	return;
}	


// This is the entry point for the graphics subsystem. For performance reasons,
// panels draw into a buffer which is displayed at once.
void callGFXSubsys ( void )
{
	int i;
	// give focus to all active panels, topmost one draws last
	for (i = 0; i < panelCount; i++)
	{
		activePanels[i] (GFX_MSG_DRAW, currentLine);
	}
	
	// dispatch to lcd driver
	dispatchBuffer();
	
	// increment current line, wrap around when everything is drawn
	currentLine ++;
	if (currentLine == LCD_LINE_COUNT)
	{
		currentLine = 0;
	}	

	// we're done here
	return;
}


void displayPanel ( uint8_t panelId )
{
	// put the new panel on top of everything
	activePanels[panelCount] = getPanelCallback ( panelId );
	// increase counter
	panelCount++;
	
	// done here
	return;
}


void dispatchBuffer ( void )
{
	int i = 0;

	// set up address for future writes
	writeLCDCtrl(0,	LCD_SET_X_ADDRESS);
	writeLCDCtrl(currentLine, LCD_SET_Y_ADDRESS);
	
	// Loop over buffer and dispatch
	for (i=0; i< LCD_LINE_LENGTH; i++)
	{
		writeLCDData(lineBuffer[i]);
	}

	// fininshed, return control
	return;
}


static panelCallbackPtr getPanelCallback (uint8_t panelId)
{
	return panelCallbacks [panelId];
}
